@@ -2,7 +2,7 @@ package PDF::Tiny;
 
 use 5.01;
 
-$VERSION = '0.05'; # Update the POD, too!
+$VERSION = '0.06'; # Update the POD, too!
 
 # Fields
 sub vers () { 0 }
@@ -127,9 +127,12 @@ sub _parse_xref {
 				: (0, $$hash{Size}[1]);
 			my ($ix, $last) = splice @indices, 0, 2;
 			$last += $ix - 1;
-			while ($stream =~ s/^(.{$width})//s) {
+			while (length $stream) {
 				my($type,$where,$gen)
-					= map hex, unpack $unpack, $1;
+					= map hex,
+					      unpack $unpack,
+					             substr $stream, 0,
+					                    $width, '';
 				$widths[0] or $type = 1;
 
 				if ($type == 1) {
@@ -168,17 +171,18 @@ sub _parse_xref {
 
 		    while ($xreftext=~ s/^$N?(\d+) (\d+).*?$N//o) {
 			for ($1..$1+$2-1) { 	
-				$xreftext =~ s/(.{20})//s; # get 20 bytes
-				my $obj_ref = "$_ " . substr($1,11,5)*1 ;
-				if (substr ($1, 17,1) eq 'n') {
+				#$xreftext =~ s/(.{20})//s; # get 20 bytes
+				my $_1 = substr($xreftext,0,20,'');
+				my $obj_ref =  "$_ " . substr($_1,11,5)*1;
+				if (substr ($_1, 17,1) eq 'n') {
 					!exists $$xref{$obj_ref}
 					  and $$xref{$obj_ref} =
-							  substr($1,0 ,10);
+							  substr($_1,0,10);
 					# (See PDF Reference [5th ed.], p. 70.)
 				}
 				else { # free
 					push @{$self->[free]}, $obj_ref
-					 unless substr($1,11,5) == 65535
+					 unless substr($_1,11,5) == 65535
 				}
 			}
 		    }
@@ -384,7 +388,8 @@ sub print {
  else { $fh = $args{fh} }
  binmode $fh;
  local ($\,$,);
- print $fh "%PDF-$self->[vers]\n%\xff\xff\xff\xff\n";
+ my $pos = length(my $buf = "%PDF-$self->[vers]\n%\xff\xff\xff\xff\n");
+ print $fh $buf;
 
  # Generate the doc ID now.  If it already exists, it might be an indirect
  # object, in which case changes to it must included in the objects that we
@@ -427,19 +432,23 @@ sub print {
    next if $seen{$id}++;
    push @ids, $id;
   }
-  $offsets{$id} = tell $fh;
+  $offsets{$id} = $pos;
   if ($$obj[0] eq 'stream') {
-   print $fh join_tokens(
+   $pos += length($buf = join_tokens(
               $id,'obj',
               @tokens
-             ), $$obj[2], "\nendstream endobj\n"
+             ) . $$obj[2] . "\nendstream endobj\n"
+           );
+   print $fh $buf;
   }
   else {
-   print $fh join_tokens(
+   $pos += length ($buf = join_tokens(
               $id,'obj',
               @tokens,
               "endobj"
-             ), "\n";
+             ) . "\n"
+           );
+   print $fh $buf;
   }
   delete $self->[objs]{$id} if $del; # Avoid reading the whole file into
  }                                   # memory at once.
@@ -455,7 +464,7 @@ sub print {
  for (0..$#free) {
   $next_free{$free[$_]} = 0+$free[$_+1];
  }
- my $startxref = tell $fh;
+ my $startxref = $pos;
  printf $fh "xref\n0 %d\n", scalar @ids;
  for (@ids) {
   printf $fh "%010d %05d %s \n",
@@ -875,17 +884,12 @@ sub tokenize { # This function tokenizes.
 			last TOKEN;
 		}
 		$prev_length = length;
-		s/^$S+//o;
-		s/^%.*?$N+//o;
-		if (s/^($R+)//o) 	{
+		s/^(?:$S++|%[^\cm\cj]*$N)+//o;
+		if (s _^(($R+)|<<|>>|[\[\]\{\}]|/$R*)__o) 	{
 			push @tokens, $1;
-			last TOKEN if defined $endtoken && $1 =~ $endtoken;
-		}
-		s/^(<<|>>)// 	and push @tokens, $1;
-		s/^([\[\]\{\}])// and push @tokens, $1;
-		s.^(<[^>]*>)..	and push @tokens, $1;
-		if (s.^(/$R*)..o) {
-	 		push @tokens, $1;
+			last TOKEN if defined $endtoken && length $2
+			           && $1 =~ $endtoken;
+			next TOKEN
 		}
 		if (s.^\(..) {  # remove paren.
 			&$more()
@@ -896,7 +900,9 @@ sub tokenize { # This function tokenizes.
 				)*\) # final closing paren
 				//xs;
 			push @tokens, "($1)";
+			next
 		}
+		s.^(<[^>]*>)..	and push @tokens, $1;
 		&$more() while /^<[^>]*\z/;
 	}
 	return @tokens;
@@ -912,7 +918,7 @@ sub join_tokens {
  for (@_) {
   # We assume that only strings can get too long to fit on a line.  After
   # all, they are the only token that can be split between lines.
-  if (/^$S*([(<])/o && length() + length $line > 254) {
+  if (length() + length $line > 254 && /^$S*([(<])/o) {
    my $hex = $1 eq '<';
    # Put a line break before the string.
    $ret .= "$line\n";
@@ -981,11 +987,9 @@ sub _interpret_token { # pass an array ref
 	# names
 
 	elsif (s.^/..) { # if it begins with "/"
-	    for (my $copy = $_) {
 		# replace #XX sequences with real chars:
 		s/#([a-f\d]{2})/chr hex $1/gie;
 		return ['name', $_];
-	    }
 	}
 
 	# dictionaries:
@@ -1091,32 +1095,6 @@ sub serialize {
 sub _serialize;
 sub _serialize {
     for (shift) {
-	# indirect references
-	$$_[0] eq 'ref' and return split(/ /,$$_[1]), "R";
-
-	# dictionaries
-	if ($$_[0] eq 'dict') {
-		my (@ret,$key,$key_copy);
-		for $key (sort keys %{$$_[1]}) {
-			($key_copy=$key)
-				=~s/($D|$S|#)/sprintf'#%02x',ord$1/ego;
-			push @ret,"/$key_copy", _serialize $$_[1]{$key};
-		}
-		return"<<",@ret,">>";
-	}
-	
-	# strings
-	if($$_[0]eq 'str'){
-		 # copy it so we don't modify the object being flattened
-		for (my $ret = $$_[1]) {
-			s/([\\()\r])/$1 eq "\r" ? '\r' : "\\$1"/ge;
-			return"($_)";
-		}
-	}
-	
-	# booleans
-	$$_[0]eq'bool'&&return+(false=>'true')[$$_[1]];
-	
 	# numbers
 	if($$_[0]eq'num'){ for ($$_[1]) {
 		!$_||$_==-0 and return 0;
@@ -1136,6 +1114,20 @@ sub _serialize {
 	    }
 	}
 	
+	# dictionaries
+	if ($$_[0] eq 'dict') {
+		my (@ret,$key,$key_copy);
+		for $key (sort keys %{$$_[1]}) {
+			($key_copy=$key)
+				=~s/($D|$S|#)/sprintf'#%02x',ord$1/ego;
+			push @ret,"/$key_copy", _serialize $$_[1]{$key};
+		}
+		return"<<",@ret,">>";
+	}
+	
+	# indirect references
+	$$_[0] eq 'ref' and return split(/ /,$$_[1]), "R";
+
 	# arrays
 	if ($$_[0]eq'array'){
 		my (@ret);
@@ -1150,11 +1142,23 @@ sub _serialize {
 		return _serialize($$_[1]), "stream\n"
 	}
 	
-	$$_[0]eq'null'&&return'null';
+	# strings
+	if($$_[0]eq 'str'){
+		 # copy it so we don't modify the object being flattened
+		for (my $ret = $$_[1]) {
+			s/([\\()\r])/$1 eq "\r" ? '\r' : "\\$1"/ge;
+			return"($_)";
+		}
+	}
+	
+	$$_[0]eq'tokens'&&return@{$$_[1]};
+	
+	# booleans
+	$$_[0]eq'bool'&&return+(false=>'true')[$$_[1]];
 	
 	$$_[0]eq'flat'&&return\$$_[1];
 	
-	$$_[0]eq'tokens'&&return@{$$_[1]};
+	$$_[0]eq'null'&&return'null';
 	
 	# If we get this far, then there's probably an empty array element or hash value which is not supposed to be there, so we shouldn't return anything.
     }
